@@ -14,9 +14,7 @@
 pub use usvg::ScreenSize;
 
 use usvg::NodeExt;
-use log::warn;
 
-#[macro_use] mod macros;
 mod clip;
 #[cfg(feature = "filter")] mod filter;
 mod image;
@@ -27,6 +25,41 @@ mod render;
 
 pub use crate::render::trim_transparency;
 
+trait OptionLog {
+    fn log_none<F: FnOnce()>(self, f: F) -> Self;
+}
+
+impl<T> OptionLog for Option<T> {
+    #[inline]
+    fn log_none<F: FnOnce()>(self, f: F) -> Self {
+        self.or_else(|| { f(); None })
+    }
+}
+
+
+trait ConvTransform {
+    fn to_native(&self) -> tiny_skia::Transform;
+    fn from_native(_: tiny_skia::Transform) -> Self;
+}
+
+impl ConvTransform for usvg::Transform {
+    fn to_native(&self) -> tiny_skia::Transform {
+        tiny_skia::Transform::from_row(
+            self.a as f32, self.b as f32,
+            self.c as f32, self.d as f32,
+            self.e as f32, self.f as f32,
+        )
+    }
+
+    fn from_native(ts: tiny_skia::Transform) -> Self {
+        Self::new(
+            ts.sx as f64, ts.ky as f64,
+            ts.kx as f64, ts.sy as f64,
+            ts.tx as f64, ts.ty as f64,
+        )
+    }
+}
+
 
 /// Renders an SVG to pixmap.
 ///
@@ -56,7 +89,7 @@ pub fn render_node(
     let node_bbox = if let Some(bbox) = node.calculate_bbox().and_then(|r| r.to_rect()) {
         bbox
     } else {
-        warn!("Node '{}' has zero size.", node.id());
+        log::warn!("Node '{}' has zero size.", node.id());
         return None;
     };
 
