@@ -2,17 +2,19 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-use crate::render::prelude::*;
+use usvg::TransformFromBBox;
+
+use crate::{ConvTransform, OptionLog, render::{Canvas, RenderState}};
 
 pub fn fill(
     tree: &usvg::Tree,
     fill: &usvg::Fill,
-    bbox: PathBbox,
+    bbox: usvg::PathBbox,
     path: &tiny_skia::Path,
     anti_alias: bool,
     blend_mode: tiny_skia::BlendMode,
     canvas: &mut Canvas,
-) {
+) -> Option<()> {
     let pattern_pixmap;
 
     let mut paint = tiny_skia::Paint::default();
@@ -35,7 +37,7 @@ pub fn fill(
                     usvg::NodeKind::Pattern(ref pattern) => {
                         let global_ts = usvg::Transform::from_native(canvas.transform);
                         let (patt_pix, patt_ts)
-                            = try_opt!(prepare_pattern_pixmap(tree, &node, pattern, &global_ts, bbox));
+                            = prepare_pattern_pixmap(tree, &node, pattern, &global_ts, bbox)?;
 
                         pattern_pixmap = patt_pix;
                         paint.shader = prepare_pattern(&pattern_pixmap, patt_ts, opacity);
@@ -56,17 +58,19 @@ pub fn fill(
     };
 
     canvas.pixmap.fill_path(path, &paint, rule, canvas.transform, canvas.clip.as_ref());
+
+    Some(())
 }
 
 pub fn stroke(
     tree: &usvg::Tree,
     stroke: &Option<usvg::Stroke>,
-    bbox: PathBbox,
+    bbox: usvg::PathBbox,
     path: &tiny_skia::Path,
     anti_alias: bool,
     blend_mode: tiny_skia::BlendMode,
     canvas: &mut Canvas,
-) {
+) -> Option<()> {
     let pattern_pixmap;
 
     let mut paint = tiny_skia::Paint::default();
@@ -91,7 +95,7 @@ pub fn stroke(
                         usvg::NodeKind::Pattern(ref pattern) => {
                             let global_ts = usvg::Transform::from_native(canvas.transform);
                             let (patt_pix, patt_ts)
-                                = try_opt!(prepare_pattern_pixmap(tree, &node, pattern, &global_ts, bbox));
+                                = prepare_pattern_pixmap(tree, &node, pattern, &global_ts, bbox)?;
 
                             pattern_pixmap = patt_pix;
                             paint.shader = prepare_pattern(&pattern_pixmap, patt_ts, opacity);
@@ -129,14 +133,16 @@ pub fn stroke(
     paint.blend_mode = blend_mode;
 
     canvas.pixmap.stroke_path(path, &paint, &props, canvas.transform, canvas.clip.as_ref());
+
+    Some(())
 }
 
 fn prepare_linear(
     g: &usvg::LinearGradient,
     opacity: usvg::Opacity,
-    bbox: PathBbox,
+    bbox: usvg::PathBbox,
     paint: &mut tiny_skia::Paint,
-) {
+) -> Option<()> {
     let mode = match g.spread_method {
         usvg::SpreadMethod::Pad => tiny_skia::SpreadMode::Pad,
         usvg::SpreadMethod::Reflect => tiny_skia::SpreadMode::Reflect,
@@ -145,8 +151,8 @@ fn prepare_linear(
 
     let transform = {
         if g.units == usvg::Units::ObjectBoundingBox {
-            let bbox = try_opt_warn_or!(bbox.to_rect(), (),
-                "Gradient on zero-sized shapes is not allowed.");
+            let bbox = bbox.to_rect()
+                .log_none(|| log::warn!("Gradient on zero-sized shapes is not allowed."))?;
 
             let mut ts = usvg::Transform::from_bbox(bbox);
             ts.append(&g.transform);
@@ -158,7 +164,7 @@ fn prepare_linear(
 
     let mut points = Vec::with_capacity(g.stops.len());
     for stop in &g.stops {
-        let alpha = stop.opacity * opacity * Opacity::new(f64::from(stop.color.alpha) / 255.0);
+        let alpha = stop.opacity * opacity * usvg::Opacity::new(f64::from(stop.color.alpha) / 255.0);
         let color = tiny_skia::Color::from_rgba8(
             stop.color.red, stop.color.green, stop.color.blue, alpha.to_u8());
         points.push(tiny_skia::GradientStop::new(stop.offset.value() as f32, color))
@@ -175,14 +181,16 @@ fn prepare_linear(
     if let Some(gradient) = gradient {
         paint.shader = gradient;
     }
+
+    Some(())
 }
 
 fn prepare_radial(
     g: &usvg::RadialGradient,
     opacity: usvg::Opacity,
-    bbox: PathBbox,
+    bbox: usvg::PathBbox,
     paint: &mut tiny_skia::Paint,
-) {
+) -> Option<()> {
     let mode = match g.spread_method {
         usvg::SpreadMethod::Pad => tiny_skia::SpreadMode::Pad,
         usvg::SpreadMethod::Reflect => tiny_skia::SpreadMode::Reflect,
@@ -191,8 +199,8 @@ fn prepare_radial(
 
     let transform = {
         if g.units == usvg::Units::ObjectBoundingBox {
-            let bbox = try_opt_warn_or!(bbox.to_rect(), (),
-                "Gradient on zero-sized shapes is not allowed.");
+            let bbox = bbox.to_rect()
+                .log_none(|| log::warn!("Gradient on zero-sized shapes is not allowed."))?;
 
             let mut ts = usvg::Transform::from_bbox(bbox);
             ts.append(&g.transform);
@@ -204,7 +212,7 @@ fn prepare_radial(
 
     let mut points = Vec::with_capacity(g.stops.len());
     for stop in &g.stops {
-        let alpha = stop.opacity * opacity * Opacity::new(f64::from(stop.color.alpha) / 255.0);
+        let alpha = stop.opacity * opacity * usvg::Opacity::new(f64::from(stop.color.alpha) / 255.0);
         let color = tiny_skia::Color::from_rgba8(
             stop.color.red, stop.color.green, stop.color.blue, alpha.to_u8());
         points.push(tiny_skia::GradientStop::new(stop.offset.value() as f32, color))
@@ -222,6 +230,8 @@ fn prepare_radial(
     if let Some(gradient) = gradient {
         paint.shader = gradient;
     }
+
+    Some(())
 }
 
 fn prepare_pattern_pixmap(
@@ -229,11 +239,11 @@ fn prepare_pattern_pixmap(
     pattern_node: &usvg::Node,
     pattern: &usvg::Pattern,
     global_ts: &usvg::Transform,
-    bbox: PathBbox,
+    bbox: usvg::PathBbox,
 ) -> Option<(tiny_skia::Pixmap, usvg::Transform)> {
     let r = if pattern.units == usvg::Units::ObjectBoundingBox {
-        let bbox = try_opt_warn_or!(bbox.to_rect(), None,
-            "Pattern on zero-sized shapes is not allowed.");
+        let bbox = bbox.to_rect()
+            .log_none(|| log::warn!("Pattern on zero-sized shapes is not allowed."))?;
 
         pattern.rect.bbox_transform(bbox)
     } else {
@@ -244,7 +254,7 @@ fn prepare_pattern_pixmap(
     ts2.append(&pattern.transform);
     let (sx, sy) = ts2.get_scale();
 
-    let img_size = Size::new(r.width() * sx, r.height() * sy)?.to_screen_size();
+    let img_size = usvg::Size::new(r.width() * sx, r.height() * sy)?.to_screen_size();
     let mut pixmap = tiny_skia::Pixmap::new(img_size.width(), img_size.height())?;
     let mut canvas = Canvas::from(pixmap.as_mut());
 
